/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package  mir.misc;

import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.Perl5Matcher;
import org.apache.oro.text.regex.Perl5Substitution;
import org.apache.oro.text.regex.Util;
import org.apache.oro.text.regex.Perl5Compiler;

import java.text.NumberFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

/**
 */
public final class StringUtil {

  private static TimeZone UTC = TimeZone.getTimeZone("UTC");

  private StringUtil() { }  // this avoids contruction

  /**
   * Formats a number with the specified minimum and maximum number of digits.
   **/
  public static synchronized String zeroPaddingNumber(long value, int minDigits,
                                                      int maxDigits)
  {
    NumberFormat numberFormat = NumberFormat.getInstance();
    numberFormat.setMinimumIntegerDigits(minDigits);
    numberFormat.setMaximumIntegerDigits(maxDigits);
    return numberFormat.format(value);
  }

  /**
   * Wandelt Datum in einen 8-ziffrigen String um (yyyymmdd)
   * @param theDate
   * @return 8-ziffriger String (yyyymmdd)
   */

  public static String date2webdbDate (GregorianCalendar theDate) {
    StringBuffer webdbDate = new StringBuffer();
    webdbDate.append(String.valueOf(theDate.get(Calendar.YEAR)));
    webdbDate.append(pad2(theDate.get(Calendar.MONTH) + 1));
    webdbDate.append(pad2(theDate.get(Calendar.DATE)));

    return  webdbDate.toString();
  }

  /**
   * Return a http://www.w3.org/TR/NOTE-datetime formatted date (yyyy-mm-ddThh:mm:ssTZ)
   * @param theDate
   * @return w3approved datetime
   */

  public static String date2w3DateTime (GregorianCalendar theDate) {
    StringBuffer webdbDate = new StringBuffer();
    webdbDate.append(String.valueOf(theDate.get(Calendar.YEAR)));
    webdbDate.append("-");
    webdbDate.append(pad2(theDate.get(Calendar.MONTH) + 1));
    webdbDate.append("-");
    webdbDate.append(pad2(theDate.get(Calendar.DATE)));
    webdbDate.append("T");
    webdbDate.append(pad2(theDate.get(Calendar.HOUR_OF_DAY)));
    webdbDate.append(":");
    webdbDate.append(pad2(theDate.get(Calendar.MINUTE)));
    webdbDate.append(":");
    webdbDate.append(pad2(theDate.get(Calendar.SECOND)));
    //assumes you are an hour-multiple away from UTC....
    int offset=(theDate.get(Calendar.ZONE_OFFSET)/(60*60*1000));
    if (offset < 0){
      webdbDate.append("-");
    }
    else{
      webdbDate.append("+");
    }
    webdbDate.append(pad2(Math.abs(offset)));
    webdbDate.append(":00");
    return  webdbDate.toString();
  }

  /**
   * wandelt Calendar in dd.mm.yyyy / hh.mm um
   * @param theDate
   * @return String mit (dd.mm.yyyy / hh.mm um)
   */
  public static String date2readableDateTime (GregorianCalendar theDate) {
    String readable = "";
    int hour;
    readable += pad2(theDate.get(Calendar.DATE));
    readable += "." + pad2(theDate.get(Calendar.MONTH) + 1);
    readable += "." + String.valueOf(theDate.get(Calendar.YEAR));
    hour = theDate.get(Calendar.HOUR);
    if (theDate.get(Calendar.AM_PM) == Calendar.PM)
      hour += 12;
    readable += " / " + pad2(hour);
    readable += ":" + pad2(theDate.get(Calendar.MINUTE));
    return  readable;
  }

  /**
   * wandelt eine Datum in einen 8-buchstabigen String, der durch <code>/</code>
   * getrennt ist.
   *
   * @param webdbDate
   * @return String mit <code>/yyyy/mm/dd</code>
   */
  public static String webdbDate2path (String webdbDate) {
    StringBuffer path = new StringBuffer();
    path.append("/").append(webdbDate.substring(0, 4));
    path.append("/").append(webdbDate.substring(4, 6));
    path.append("/");
    //who did this?
    //path.append("/").append(webdbDate.substring(6, 8));
    return  path.toString();
  }

  /**
   * L?scht <code>/</code> am Ende des Strings, falls vorhanden
   * @param path
   * @return String ohne <code>/</code> am Ende
   */
  public static String removeSlash (String path) {
    return  path.length() > 1 && path.endsWith("/") ? path.substring(0, path.length()
        - 1) : path;
  }

  /**
   * formatiert eine Zahl (0-99) zweistellig (z.B. 5 -> 05)
   * @return zwistellige Zahl
   */
  public static String pad2 (int number) {
    return  number < 10 ? "0" + number : String.valueOf(number);
  }

  /**
   * formatiert eine Zahl (0-999) dreistellig (z.B. 7 -> 007)
   *
   * @return 3-stellige Zahl
   */
  public static String pad3 (int number) {
    return  number < 10 ? "00" + number : number < 100 ? "0" + number : String.valueOf(number);
  }

  /**
   * Liefert Default-Wert def zur?ck, wenn String <code>s</code>
   * kein Integer ist.
   *
   * @param s
   * @param def
   * @return geparster int aus s oder def
   */
  public static int parseInt(String s, int def) {
    if (s == null) return def;
    try {
      return Integer.parseInt(s);
    } catch (NumberFormatException e) {
      return def;
    }
  }


  /**
   * Converts mir's horrible internal date format (yyyy-MM-dd HH:mm:ss+zz) into a java Date
   *
   * @param anInternalDate
   */
  public static Date convertMirInternalDateToDate(String anInternalDate) {
    Calendar calendar = new GregorianCalendar();

    int year;
    int month;
    int day;
    int hours;
    int minutes;
    int seconds;
    int timezoneOffset;

    year = Integer.parseInt(anInternalDate.substring(0,4));
    month = Integer.parseInt(anInternalDate.substring(5,7));
    day = Integer.parseInt(anInternalDate.substring(8,10));
    hours = Integer.parseInt(anInternalDate.substring(11,13));
    minutes = Integer.parseInt(anInternalDate.substring(14,16));
    seconds = Integer.parseInt(anInternalDate.substring(17,19));

    timezoneOffset = Integer.parseInt(anInternalDate.substring(20,22));
    if (anInternalDate.charAt(19) == '-') {
      timezoneOffset = -timezoneOffset;
    }

    calendar.setTimeZone(UTC);
    calendar.set(year, month-1, day, hours, minutes, seconds);
    calendar.add(Calendar.HOUR, -timezoneOffset);

    return calendar.getTime();
  }

  public String regexpReplace(String anInput, String anExpression, String aReplacement) throws Exception {
    Perl5Matcher matcher = new Perl5Matcher();

    return Util.substitute(
      matcher, new Perl5Compiler().compile(anExpression),
      new Perl5Substitution(aReplacement), anInput,
      Util.SUBSTITUTE_ALL);
  }



}

